@@ -283,7 +283,7 @@ gst_gl_image_sink_bin_class_init (GstGLImageSinkBinClass * klass)
       g_param_spec_enum ("output-multiview-downmix-mode",
           "Mode for mono downmixed output",
           "Output anaglyph type to generate when downmixing to mono",
-          GST_TYPE_GL_STEREO_DOWNMIX_MODE_TYPE, DEFAULT_MULTIVIEW_DOWNMIX,
+          GST_TYPE_GL_STEREO_DOWNMIX, DEFAULT_MULTIVIEW_DOWNMIX,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gst_video_overlay_install_properties (gobject_class, PROP_BIN_LAST);
@@ -694,7 +694,7 @@ gst_glimage_sink_class_init (GstGLImageSinkClass * klass)
       g_param_spec_enum ("output-multiview-downmix-mode",
           "Mode for mono downmixed output",
           "Output anaglyph type to generate when downmixing to mono",
-          GST_TYPE_GL_STEREO_DOWNMIX_MODE_TYPE, DEFAULT_MULTIVIEW_DOWNMIX,
+          GST_TYPE_GL_STEREO_DOWNMIX, DEFAULT_MULTIVIEW_DOWNMIX,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gst_video_overlay_install_properties (gobject_class, PROP_LAST);
@@ -2014,10 +2014,17 @@ gst_glimage_sink_thread_init_redisplay (GstGLImageSink * gl_sink)
       GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY,
       gst_gl_shader_string_vertex_mat4_vertex_transform);
   if (gl_sink->texture_target == GST_GL_TEXTURE_TARGET_EXTERNAL_OES) {
+    gchar *frag_str;
+    frag_str =
+        gst_gl_shader_string_fragment_external_oes_get_default
+        (gl_sink->context, GST_GLSL_VERSION_NONE,
+        GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY);
+
     frag_stage = gst_glsl_stage_new_with_string (gl_sink->context,
         GL_FRAGMENT_SHADER, GST_GLSL_VERSION_NONE,
-        GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY,
-        gst_gl_shader_string_fragment_external_oes_default);
+        GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY, frag_str);
+
+    g_free (frag_str);
   } else {
     frag_stage = gst_glsl_stage_new_default_fragment (gl_sink->context);
   }
@@ -2180,6 +2187,16 @@ gst_glimage_sink_on_resize (GstGLImageSink * gl_sink, gint width, gint height)
     GST_DEBUG_OBJECT (gl_sink, "GL output area now %u,%u %ux%u",
         gl_sink->display_rect.x, gl_sink->display_rect.y,
         gl_sink->display_rect.w, gl_sink->display_rect.h);
+  } else {
+    gint viewport_dims[4];
+
+    /* save the viewport for use later */
+    gl->GetIntegerv (GL_VIEWPORT, viewport_dims);
+
+    gl_sink->display_rect.x = viewport_dims[0];
+    gl_sink->display_rect.y = viewport_dims[1];
+    gl_sink->display_rect.w = viewport_dims[2];
+    gl_sink->display_rect.h = viewport_dims[3];
   }
   GST_GLIMAGE_SINK_UNLOCK (gl_sink);
 }
@@ -2227,6 +2244,14 @@ gst_glimage_sink_on_draw (GstGLImageSink * gl_sink)
   gst_gl_context_clear_shader (gl_sink->context);
   gl->BindTexture (gl_target, 0);
 
+  if (!gst_gl_window_controls_viewport (window)) {
+    gl->Viewport (gl_sink->display_rect.x, gl_sink->display_rect.y,
+        gl_sink->display_rect.w, gl_sink->display_rect.h);
+    GST_DEBUG_OBJECT (gl_sink, "GL output area now %u,%u %ux%u",
+        gl_sink->display_rect.x, gl_sink->display_rect.y,
+        gl_sink->display_rect.w, gl_sink->display_rect.h);
+  }
+
   sample = gst_sample_new (gl_sink->stored_buffer[0],
       gl_sink->out_caps, &GST_BASE_SINK (gl_sink)->segment, NULL);
   g_signal_emit (gl_sink, gst_glimage_sink_signals[CLIENT_DRAW_SIGNAL], 0,
@@ -2249,7 +2274,14 @@ gst_glimage_sink_on_draw (GstGLImageSink * gl_sink)
 
     if (gl_sink->ignore_alpha) {
       gl->BlendColor (0.0, 0.0, 0.0, alpha);
-      gl->BlendFunc (GL_SRC_ALPHA, GL_CONSTANT_COLOR);
+      if (gl->BlendFuncSeparate) {
+        gl->BlendFuncSeparate (GL_SRC_ALPHA, GL_CONSTANT_COLOR, GL_ONE,
+            GL_ONE_MINUS_SRC_ALPHA);
+      } else {
+        /* we don't have separate blend modes, perform something close to
+         * correct instead */
+        gl->BlendFunc (GL_SRC_ALPHA, GL_CONSTANT_COLOR);
+      }
       gl->BlendEquation (GL_FUNC_ADD);
       gl->Enable (GL_BLEND);
     }
@@ -2291,7 +2323,8 @@ gst_glimage_sink_on_draw (GstGLImageSink * gl_sink)
 
     if (gl->GenVertexArrays)
       gl->BindVertexArray (0);
-    _unbind_buffer (gl_sink);
+    else
+      _unbind_buffer (gl_sink);
 
     if (gl_sink->ignore_alpha)
       gl->Disable (GL_BLEND);
