@@ -307,10 +307,10 @@ gst_gl_sink_bin_set_sink (GstGLSinkBin * self, GstElement * sink)
   }
   self->sink = sink;
 
-  if (sink && g_object_is_floating (sink))
-    gst_object_ref_sink (sink);
+  gst_object_ref_sink (sink);
 
   if (sink && !_connect_sink_element (self)) {
+    gst_object_unref (self->sink);
     self->sink = NULL;
     return FALSE;
   }
@@ -322,8 +322,7 @@ void
 gst_gl_sink_bin_finish_init_with_element (GstGLSinkBin * self,
     GstElement * element)
 {
-  if (!gst_gl_sink_bin_set_sink (self, element))
-    gst_object_unref (element);
+  gst_gl_sink_bin_set_sink (self, element);
 }
 
 void
@@ -344,6 +343,7 @@ gst_gl_sink_bin_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
 {
   GstGLSinkBin *self = GST_GL_SINK_BIN (object);
+  GParamSpec *sink_pspec;
 
   switch (prop_id) {
     case PROP_SINK:
@@ -357,8 +357,17 @@ gst_gl_sink_bin_set_property (GObject * object, guint prop_id,
         g_object_set_property (G_OBJECT (self->balance), pspec->name, value);
       break;
     default:
-      if (self->sink)
-        g_object_set_property (G_OBJECT (self->sink), pspec->name, value);
+      if (self->sink) {
+        sink_pspec =
+            g_object_class_find_property (G_OBJECT_GET_CLASS (self->sink),
+            pspec->name);
+        if (sink_pspec
+            && G_PARAM_SPEC_TYPE (sink_pspec) == G_PARAM_SPEC_TYPE (pspec)) {
+          g_object_set_property (G_OBJECT (self->sink), pspec->name, value);
+        } else {
+          GST_INFO ("Failed to set unmatched property %s", pspec->name);
+        }
+      }
       break;
   }
 }
