@@ -172,8 +172,6 @@ gst_time_overlay_class_init (GstTimeOverlayClass * klass)
   GstElementClass *gstelement_class;
   GstBaseTextOverlayClass *gsttextoverlay_class;
   GObjectClass *gobject_class;
-  PangoContext *context;
-  PangoFontDescription *font_description;
 
   gsttextoverlay_class = (GstBaseTextOverlayClass *) klass;
   gstelement_class = (GstElementClass *) klass;
@@ -193,9 +191,23 @@ gst_time_overlay_class_init (GstTimeOverlayClass * klass)
       g_param_spec_enum ("time-mode", "Time Mode", "What time to show",
           GST_TYPE_TIME_OVERLAY_TIME_LINE, DEFAULT_TIME_LINE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+}
 
-  g_mutex_lock (gsttextoverlay_class->pango_lock);
-  context = gsttextoverlay_class->pango_context;
+static void
+gst_time_overlay_init (GstTimeOverlay * overlay)
+{
+  GstBaseTextOverlay *textoverlay;
+  PangoContext *context;
+  PangoFontDescription *font_description;
+
+  textoverlay = GST_BASE_TEXT_OVERLAY (overlay);
+
+  textoverlay->valign = GST_BASE_TEXT_OVERLAY_VALIGN_TOP;
+  textoverlay->halign = GST_BASE_TEXT_OVERLAY_HALIGN_LEFT;
+
+  overlay->time_line = DEFAULT_TIME_LINE;
+
+  context = textoverlay->pango_context;
 
   pango_context_set_language (context, pango_language_from_string ("en_US"));
   pango_context_set_base_dir (context, PANGO_DIRECTION_LTR);
@@ -209,20 +221,6 @@ gst_time_overlay_class_init (GstTimeOverlayClass * klass)
   pango_font_description_set_size (font_description, 18 * PANGO_SCALE);
   pango_context_set_font_description (context, font_description);
   pango_font_description_free (font_description);
-  g_mutex_unlock (gsttextoverlay_class->pango_lock);
-}
-
-static void
-gst_time_overlay_init (GstTimeOverlay * overlay)
-{
-  GstBaseTextOverlay *textoverlay;
-
-  textoverlay = GST_BASE_TEXT_OVERLAY (overlay);
-
-  textoverlay->valign = GST_BASE_TEXT_OVERLAY_VALIGN_TOP;
-  textoverlay->halign = GST_BASE_TEXT_OVERLAY_HALIGN_LEFT;
-
-  overlay->time_line = DEFAULT_TIME_LINE;
 }
 
 static void
