@@ -755,29 +755,43 @@ gst_audio_aggregator_sink_setcaps (GstAudioAggregatorPad * aaggpad,
 {
   GstAudioAggregatorPad *first_configured_pad =
       gst_audio_aggregator_get_first_configured_pad (agg);
-  GstCaps *downstream_caps = gst_pad_get_allowed_caps (agg->srcpad);
   GstAudioInfo info;
   gboolean ret = TRUE;
-  gint downstream_rate;
-  GstStructure *s;
-
-  if (!downstream_caps || gst_caps_is_empty (downstream_caps)) {
-    ret = FALSE;
-    goto done;
-  }
+  gboolean downstream_supports_rate = TRUE;
 
   if (!gst_audio_info_from_caps (&info, caps)) {
     GST_WARNING_OBJECT (agg, "Rejecting invalid caps: %" GST_PTR_FORMAT, caps);
     return FALSE;
   }
-  s = gst_caps_get_structure (downstream_caps, 0);
 
   /* TODO: handle different rates on sinkpads, a bit complex
    * because offsets will have to be updated, and audio resampling
    * has a latency to take into account
    */
-  if ((gst_structure_get_int (s, "rate", &downstream_rate)
-          && info.rate != downstream_rate) || (first_configured_pad
+
+  /* Only check against the downstream caps if we didn't configure any caps
+   * so far. Otherwise we already know that downstream supports the rate
+   * because we negotiated with downstream */
+  if (!first_configured_pad) {
+    GstCaps *downstream_caps = gst_pad_get_allowed_caps (agg->srcpad);
+
+    /* Returns NULL if there is no downstream peer */
+    if (downstream_caps) {
+      GstCaps *rate_caps =
+          gst_caps_new_simple ("audio/x-raw", "rate", G_TYPE_INT, info.rate,
+          NULL);
+
+      gst_caps_set_features_simple (rate_caps,
+          gst_caps_features_copy (GST_CAPS_FEATURES_ANY));
+
+      downstream_supports_rate =
+          gst_caps_can_intersect (rate_caps, downstream_caps);
+      gst_caps_unref (rate_caps);
+      gst_caps_unref (downstream_caps);
+    }
+  }
+
+  if (!downstream_supports_rate || (first_configured_pad
           && info.rate != first_configured_pad->info.rate)) {
     gst_pad_push_event (GST_PAD (aaggpad), gst_event_new_reconfigure ());
     ret = FALSE;
@@ -791,13 +805,9 @@ gst_audio_aggregator_sink_setcaps (GstAudioAggregatorPad * aaggpad,
     GST_OBJECT_UNLOCK (aaggpad);
   }
 
-done:
   if (first_configured_pad)
     gst_object_unref (first_configured_pad);
 
-  if (downstream_caps)
-    gst_caps_unref (downstream_caps);
-
   return ret;
 }
 
