@@ -62,16 +62,9 @@
 /*                                                              */
 /* =============================================================*/
 
-#define GST_GL_WINDOW_COCOA_GET_PRIVATE(o)  \
-  (G_TYPE_INSTANCE_GET_PRIVATE((o), GST_TYPE_GL_WINDOW_COCOA, GstGLWindowCocoaPrivate))
-
 #define GST_CAT_DEFAULT gst_gl_window_cocoa_debug
 GST_DEBUG_CATEGORY_STATIC (GST_CAT_DEFAULT);
 
-#define DEBUG_INIT \
-  GST_DEBUG_CATEGORY_GET (GST_CAT_DEFAULT, "glwindow");
-#define gst_gl_window_cocoa_parent_class parent_class
-G_DEFINE_TYPE_WITH_CODE (GstGLWindowCocoa, gst_gl_window_cocoa, GST_TYPE_GL_WINDOW, DEBUG_INIT);
 static void gst_gl_window_cocoa_finalize (GObject * object);
 
 static gboolean gst_gl_window_cocoa_open (GstGLWindow *window, GError **err);
@@ -88,6 +81,8 @@ static void gst_gl_window_cocoa_send_message_async (GstGLWindow * window,
     GstGLWindowCB callback, gpointer data, GDestroyNotify destroy);
 static gboolean gst_gl_window_cocoa_set_render_rectangle (GstGLWindow * window,
     gint x, gint y, gint width, gint height);
+static gboolean gst_gl_window_cocoa_controls_viewport (GstGLWindow * window);
+
 
 struct _GstGLWindowCocoaPrivate
 {
@@ -103,14 +98,20 @@ struct _GstGLWindowCocoaPrivate
   gpointer gl_queue;
 };
 
+#define DEBUG_INIT \
+  GST_DEBUG_CATEGORY_GET (GST_CAT_DEFAULT, "glwindow");
+
+#define gst_gl_window_cocoa_parent_class parent_class
+G_DEFINE_TYPE_WITH_CODE (GstGLWindowCocoa, gst_gl_window_cocoa, GST_TYPE_GL_WINDOW,
+    G_ADD_PRIVATE (GstGLWindowCocoa)
+    DEBUG_INIT);
+
 static void
 gst_gl_window_cocoa_class_init (GstGLWindowCocoaClass * klass)
 {
   GstGLWindowClass *window_class = (GstGLWindowClass *) klass;
   GObjectClass *gobject_class = (GObjectClass *) klass;
 
-  g_type_class_add_private (klass, sizeof (GstGLWindowCocoaPrivate));
-
   window_class->open = GST_DEBUG_FUNCPTR (gst_gl_window_cocoa_open);
   window_class->close = GST_DEBUG_FUNCPTR (gst_gl_window_cocoa_close);
   window_class->get_window_handle =
@@ -126,6 +127,8 @@ gst_gl_window_cocoa_class_init (GstGLWindowCocoaClass * klass)
       GST_DEBUG_FUNCPTR (gst_gl_window_cocoa_send_message_async);
   window_class->set_render_rectangle =
       GST_DEBUG_FUNCPTR (gst_gl_window_cocoa_set_render_rectangle);
+  window_class->controls_viewport =
+      GST_DEBUG_FUNCPTR (gst_gl_window_cocoa_controls_viewport);
 
   gobject_class->finalize = gst_gl_window_cocoa_finalize;
 }
@@ -133,7 +136,7 @@ gst_gl_window_cocoa_class_init (GstGLWindowCocoaClass * klass)
 static void
 gst_gl_window_cocoa_init (GstGLWindowCocoa * window)
 {
-  window->priv = GST_GL_WINDOW_COCOA_GET_PRIVATE (window);
+  window->priv = gst_gl_window_cocoa_get_instance_private (window);
 
   window->priv->preferred_width = 320;
   window->priv->preferred_height = 240;
@@ -237,16 +240,24 @@ gst_gl_window_cocoa_open (GstGLWindow *window, GError **err)
 }
 
 static void
-gst_gl_window_cocoa_close (GstGLWindow *window)
+_close_window (gpointer * data)
 {
-  GstGLWindowCocoa *window_cocoa = GST_GL_WINDOW_COCOA (window);
-  GstGLNSWindow *internal_win_id = (__bridge GstGLNSWindow *)window_cocoa->priv->internal_win_id;
+  GstGLWindowCocoa *window_cocoa = GST_GL_WINDOW_COCOA (data);
+  GstGLNSWindow *internal_win_id =
+      (__bridge GstGLNSWindow *) window_cocoa->priv->internal_win_id;
 
   [[internal_win_id contentView] removeFromSuperview];
-  CFBridgingRelease(window_cocoa->priv->internal_win_id);
+  CFBridgingRelease (window_cocoa->priv->internal_win_id);
   window_cocoa->priv->internal_win_id = NULL;
 }
 
+static void
+gst_gl_window_cocoa_close (GstGLWindow * window)
+{
+  _invoke_on_main ((GstGLWindowCB) _close_window, gst_object_ref (window),
+      (GDestroyNotify) gst_object_unref);
+}
+
 static guintptr
 gst_gl_window_cocoa_get_window_handle (GstGLWindow *window)
 {
@@ -388,9 +399,11 @@ gst_gl_cocoa_draw_cb (GstGLWindowCocoa *window_cocoa)
   GstGLNSWindow *internal_win_id = (__bridge GstGLNSWindow *)priv->internal_win_id;
 
   if (internal_win_id && ![internal_win_id isClosed]) {
-   GstGLWindow *window = GST_GL_WINDOW (window_cocoa);
+    GstGLWindow *window = GST_GL_WINDOW (window_cocoa);
 
     /* draw opengl scene in the back buffer */
+    /* We do not need to change viewports like in other window implementations
+     * as the caopengllayer will take care of that for us. */
     if (window->draw)
       window->draw (window->draw_data);
   }
@@ -410,6 +423,7 @@ gst_gl_cocoa_resize_cb (GstGLNSView * view, guint width, guint height)
     NSRect bounds = [view bounds];
     NSRect visibleRect = [view visibleRect];
     gint viewport_dim[4];
+    GstVideoRectangle viewport;
 
     gl = context->gl_vtable;
 
@@ -418,19 +432,30 @@ gst_gl_cocoa_resize_cb (GstGLNSView * view, guint width, guint height)
     visibleRect = [view convertRectToBacking:visibleRect];
 #endif
 
+    /* don't use the default gst_gl_window_resize() as that will marshal through
+     * the GL thread.  We are being called from the main thread by the
+     * caopengllayer */
+    if (window->resize)
+      window->resize (window->resize_data, width, height);
+
+    gl->GetIntegerv (GL_VIEWPORT, viewport_dim);
+
     GST_DEBUG_OBJECT (window, "Window resized: bounds %lf %lf %lf %lf "
-                      "visibleRect %lf %lf %lf %lf",
+                      "visibleRect %lf %lf %lf %lf, "
+                      "viewport dimensions %i %i %i %i",
                       bounds.origin.x, bounds.origin.y,
                       bounds.size.width, bounds.size.height,
                       visibleRect.origin.x, visibleRect.origin.y,
-                      visibleRect.size.width, visibleRect.size.height);
+                      visibleRect.size.width, visibleRect.size.height,
+                      viewport_dim[0], viewport_dim[1], viewport_dim[2],
+                      viewport_dim[3]);
 
-    gst_gl_window_resize (window, width, height);
-    gl->GetIntegerv (GL_VIEWPORT, viewport_dim);
+    viewport.x = viewport_dim[0] - visibleRect.origin.x;
+    viewport.x = viewport_dim[1] - visibleRect.origin.y;
+    viewport.w = viewport_dim[2];
+    viewport.h = viewport_dim[3];
 
-    gl->Viewport (viewport_dim[0] - visibleRect.origin.x,
-                  viewport_dim[1] - visibleRect.origin.y,
-                  viewport_dim[2], viewport_dim[3]);
+    gl->Viewport (viewport.x, viewport.y, viewport.w, viewport.h);
   }
 
   gst_object_unref (context);
@@ -533,6 +558,12 @@ gst_gl_window_cocoa_set_render_rectangle (GstGLWindow * window, gint x, gint y,
  return TRUE;
 }
 
+static gboolean
+gst_gl_window_cocoa_controls_viewport (GstGLWindow * window)
+{
+  return TRUE;
+}
+
 /* =============================================================*/
 /*                                                              */
 /*                    GstGLNSWindow implementation              */
