@@ -81,16 +81,6 @@ GST_DEBUG_CATEGORY_STATIC (gst_context);
 GST_DEBUG_CATEGORY_STATIC (gst_gl_display_debug);
 #define GST_CAT_DEFAULT gst_gl_display_debug
 
-#define DEBUG_INIT \
-  GST_DEBUG_CATEGORY_INIT (gst_gl_display_debug, "gldisplay", 0, "opengl display"); \
-  GST_DEBUG_CATEGORY_GET (gst_context, "GST_CONTEXT");
-
-G_DEFINE_TYPE_WITH_CODE (GstGLDisplay, gst_gl_display, GST_TYPE_OBJECT,
-    DEBUG_INIT);
-
-#define GST_GL_DISPLAY_GET_PRIVATE(o) \
-  (G_TYPE_INSTANCE_GET_PRIVATE((o), GST_TYPE_GL_DISPLAY, GstGLDisplayPrivate))
-
 enum
 {
   SIGNAL_0,
@@ -119,6 +109,14 @@ struct _GstGLDisplayPrivate
   GCond thread_cond;
 };
 
+#define DEBUG_INIT \
+  GST_DEBUG_CATEGORY_INIT (gst_gl_display_debug, "gldisplay", 0, "opengl display"); \
+  GST_DEBUG_CATEGORY_GET (gst_context, "GST_CONTEXT");
+
+G_DEFINE_TYPE_WITH_CODE (GstGLDisplay, gst_gl_display, GST_TYPE_OBJECT,
+    G_ADD_PRIVATE (GstGLDisplay)
+    DEBUG_INIT);
+
 static gboolean
 _unlock_main_thread (GstGLDisplay * display)
 {
@@ -158,8 +156,6 @@ _event_thread_main (GstGLDisplay * display)
 static void
 gst_gl_display_class_init (GstGLDisplayClass * klass)
 {
-  g_type_class_add_private (klass, sizeof (GstGLDisplayPrivate));
-
   /**
    * GstGLDisplay::create-context:
    * @object: the #GstGLDisplay
@@ -186,7 +182,7 @@ gst_gl_display_class_init (GstGLDisplayClass * klass)
 static void
 gst_gl_display_init (GstGLDisplay * display)
 {
-  display->priv = GST_GL_DISPLAY_GET_PRIVATE (display);
+  display->priv = gst_gl_display_get_instance_private (display);
 
   display->type = GST_GL_DISPLAY_TYPE_ANY;
   display->priv->gl_api = GST_GL_API_ANY;
