@@ -78,12 +78,6 @@
 #define GST_CAT_DEFAULT gst_gl_window_debug
 GST_DEBUG_CATEGORY (GST_CAT_DEFAULT);
 
-#define gst_gl_window_parent_class parent_class
-G_DEFINE_ABSTRACT_TYPE (GstGLWindow, gst_gl_window, GST_TYPE_OBJECT);
-
-#define GST_GL_WINDOW_GET_PRIVATE(o) \
-  (G_TYPE_INSTANCE_GET_PRIVATE((o), GST_TYPE_GL_WINDOW, GstGLWindowPrivate))
-
 static void gst_gl_window_default_draw (GstGLWindow * window);
 static void gst_gl_window_default_run (GstGLWindow * window);
 static void gst_gl_window_default_quit (GstGLWindow * window);
@@ -105,6 +99,10 @@ struct _GstGLWindowPrivate
   GCond sync_message_cond;
 };
 
+#define gst_gl_window_parent_class parent_class
+G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GstGLWindow, gst_gl_window,
+    GST_TYPE_OBJECT);
+
 static void gst_gl_window_finalize (GObject * object);
 
 typedef struct _GstGLDummyWindow
@@ -163,7 +161,7 @@ _init_debug (void)
 static void
 gst_gl_window_init (GstGLWindow * window)
 {
-  GstGLWindowPrivate *priv = GST_GL_WINDOW_GET_PRIVATE (window);
+  GstGLWindowPrivate *priv = gst_gl_window_get_instance_private (window);
   window->priv = priv;
 
   g_mutex_init (&window->lock);
@@ -181,8 +179,6 @@ gst_gl_window_init (GstGLWindow * window)
 static void
 gst_gl_window_class_init (GstGLWindowClass * klass)
 {
-  g_type_class_add_private (klass, sizeof (GstGLWindowPrivate));
-
   klass->open = GST_DEBUG_FUNCPTR (gst_gl_window_default_open);
   klass->close = GST_DEBUG_FUNCPTR (gst_gl_window_default_close);
   klass->run = GST_DEBUG_FUNCPTR (gst_gl_window_default_run);
@@ -930,6 +926,12 @@ gst_gl_window_set_render_rectangle (GstGLWindow * window, gint x, gint y,
   return ret;
 }
 
+/**
+ * gst_gl_window_queue_resize:
+ * @window: a #GstGLWindow
+ *
+ * Queue resizing of @window.
+ */
 void
 gst_gl_window_queue_resize (GstGLWindow * window)
 {
@@ -958,6 +960,14 @@ _on_resize (gpointer data)
       resize->height);
 }
 
+/**
+ * gst_gl_window_resize:
+ * @window: a #GstGLWindow
+ * @width: new width
+ * @height: new height
+ *
+ * Resize @window to the given @width and @height.
+ */
 void
 gst_gl_window_resize (GstGLWindow * window, guint width, guint height)
 {
@@ -979,6 +989,30 @@ gst_gl_window_resize (GstGLWindow * window, guint width, guint height)
   window->queue_resize = FALSE;
 }
 
+/**
+ * gst_gl_window_controls_viewport:
+ * @window: a #GstGLWindow
+ *
+ * Checks if @window controls the GL viewport.
+ *
+ * Returns: %TRUE if @window controls the GL viewport, otherwise %FALSE
+ *
+ * Since: 1.16
+ */
+gboolean
+gst_gl_window_controls_viewport (GstGLWindow * window)
+{
+  GstGLWindowClass *window_class;
+
+  g_return_val_if_fail (GST_IS_GL_WINDOW (window), FALSE);
+  window_class = GST_GL_WINDOW_GET_CLASS (window);
+
+  if (!window_class->controls_viewport)
+    return FALSE;
+
+  return window_class->controls_viewport (window);
+}
+
 static GType gst_gl_dummy_window_get_type (void);
 
 G_DEFINE_TYPE (GstGLDummyWindow, gst_gl_dummy_window, GST_TYPE_GL_WINDOW);
