@@ -31,6 +31,9 @@
  * and get session bandwidth information.
  *
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <string.h>
 
