@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 
@@ -79,8 +79,13 @@ struct _GstVideoFilterClass {
   gpointer _gst_reserved[GST_PADDING];
 };
 
+GST_VIDEO_API
 GType gst_video_filter_get_type (void);
 
+#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstVideoFilter, gst_object_unref)
+#endif
+
 G_END_DECLS
 
 #endif /* __GST_VIDEO_FILTER_H__ */
