@@ -63,13 +63,6 @@ typedef struct
 
 #define GST_VIDEO_COLORIMETRY_NONAME  NULL
 
-#define DEFAULT_YUV_SD  0
-#define DEFAULT_YUV_HD  1
-#define DEFAULT_RGB     3
-#define DEFAULT_YUV_UHD 4
-#define DEFAULT_GRAY    5
-#define DEFAULT_UNKNOWN 6
-
 static const ColorimetryInfo colorimetry[] = {
   MAKE_COLORIMETRY (BT601, _16_235, BT601, BT709, SMPTE170M),
   MAKE_COLORIMETRY (BT709, _16_235, BT709, BT709, BT709),
@@ -77,9 +70,11 @@ static const ColorimetryInfo colorimetry[] = {
   MAKE_COLORIMETRY (SRGB, _0_255, RGB, SRGB, BT709),
   MAKE_COLORIMETRY (BT2020, _16_235, BT2020, BT2020_12, BT2020),
   MAKE_COLORIMETRY (NONAME, _0_255, BT601, UNKNOWN, UNKNOWN),
-  MAKE_COLORIMETRY (NONAME, _UNKNOWN, UNKNOWN, UNKNOWN, UNKNOWN),
+  MAKE_COLORIMETRY (NONAME, _UNKNOWN, UNKNOWN, UNKNOWN, UNKNOWN),       /* Keep last! */
 };
 
+#define DEFAULT_UNKNOWN (G_N_ELEMENTS(colorimetry)-1)
+
 static const ColorimetryInfo *
 gst_video_get_colorimetry (const gchar * s)
 {
@@ -109,7 +104,7 @@ gst_video_get_colorimetry (const gchar * s)
  * Parse the colorimetry string and update @cinfo with the parsed
  * values.
  *
- * Returns: #TRUE if @color points to valid colorimetry info.
+ * Returns: %TRUE if @color points to valid colorimetry info.
  */
 gboolean
 gst_video_colorimetry_from_string (GstVideoColorimetry * cinfo,
@@ -118,7 +113,10 @@ gst_video_colorimetry_from_string (GstVideoColorimetry * cinfo,
   const ColorimetryInfo *ci;
   gboolean res = FALSE;
 
-  if ((ci = gst_video_get_colorimetry (color))) {
+  if (!color) {
+    *cinfo = colorimetry[DEFAULT_UNKNOWN].color;
+    res = TRUE;
+  } else if ((ci = gst_video_get_colorimetry (color))) {
     *cinfo = ci->color;
     res = TRUE;
   } else {
@@ -144,7 +142,7 @@ gst_video_colorimetry_from_string (GstVideoColorimetry * cinfo,
  * Returns: a string representation of @cinfo.
  */
 gchar *
-gst_video_colorimetry_to_string (GstVideoColorimetry * cinfo)
+gst_video_colorimetry_to_string (const GstVideoColorimetry * cinfo)
 {
   gint i;
 
@@ -168,11 +166,12 @@ gst_video_colorimetry_to_string (GstVideoColorimetry * cinfo)
  * Check if the colorimetry information in @info matches that of the
  * string @color.
  *
- * Returns: #TRUE if @color conveys the same colorimetry info as the color
+ * Returns: %TRUE if @color conveys the same colorimetry info as the color
  * information in @info.
  */
 gboolean
-gst_video_colorimetry_matches (GstVideoColorimetry * cinfo, const gchar * color)
+gst_video_colorimetry_matches (const GstVideoColorimetry * cinfo,
+    const gchar * color)
 {
   const ColorimetryInfo *ci;
 
@@ -186,8 +185,8 @@ gst_video_colorimetry_matches (GstVideoColorimetry * cinfo, const gchar * color)
  * gst_video_color_range_offsets:
  * @range: a #GstVideoColorRange
  * @info: a #GstVideoFormatInfo
- * @offset: (out): output offsets
- * @scale: (out): output scale
+ * @offset: (out caller-allocates) (array fixed-size=4): output offsets
+ * @scale: (out caller-allocates) (array fixed-size=4): output scale
  *
  * Compute the offset and scale values for each component of @info. For each
  * component, (c[i] - offset[i]) / scale[i] will scale the component c[i] to the
@@ -252,7 +251,7 @@ gst_video_color_range_offsets (GstVideoColorRange range,
  *
  * Compare the 2 colorimetry sets for equality
  *
- * Returns: #TRUE if @cinfo and @other are equal.
+ * Returns: %TRUE if @cinfo and @other are equal.
  *
  * Since: 1.6
  */
@@ -268,6 +267,8 @@ gst_video_colorimetry_is_equal (const GstVideoColorimetry * cinfo,
 
 #define WP_C    0.31006, 0.31616
 #define WP_D65  0.31271, 0.32902
+#define WP_CENTRE (1/3), (1/3)
+#define WP_WHITE 0.314, 0.351
 
 static const GstVideoColorPrimariesInfo color_primaries[] = {
   {GST_VIDEO_COLOR_PRIMARIES_UNKNOWN, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
@@ -284,7 +285,15 @@ static const GstVideoColorPrimariesInfo color_primaries[] = {
   {GST_VIDEO_COLOR_PRIMARIES_BT2020, WP_D65, 0.708, 0.292, 0.170, 0.797, 0.131,
       0.046},
   {GST_VIDEO_COLOR_PRIMARIES_ADOBERGB, WP_D65, 0.64, 0.33, 0.21, 0.71, 0.15,
-      0.06}
+      0.06},
+  {GST_VIDEO_COLOR_PRIMARIES_SMPTEST428, WP_CENTRE, 1.0, 0.0, 0.0, 1.0, 0.0,
+      0.0},
+  {GST_VIDEO_COLOR_PRIMARIES_SMPTERP431, WP_WHITE, 0.68, 0.32, 0.265, 0.69,
+      0.15, 0.06},
+  {GST_VIDEO_COLOR_PRIMARIES_SMPTEEG432, WP_D65, 0.68, 0.32, 0.265, 0.69, 0.15,
+      0.06},
+  {GST_VIDEO_COLOR_PRIMARIES_EBU3213, WP_D65, 0.63, 0.34, 0.295, 0.605, 0.155,
+      0.077},
 };
 
 /**
@@ -300,8 +309,8 @@ static const GstVideoColorPrimariesInfo color_primaries[] = {
 const GstVideoColorPrimariesInfo *
 gst_video_color_primaries_get_info (GstVideoColorPrimaries primaries)
 {
-  g_return_val_if_fail (primaries <
-      (GstVideoColorPrimaries) G_N_ELEMENTS (color_primaries), NULL);
+  g_return_val_if_fail ((gint) primaries <
+      G_N_ELEMENTS (color_primaries), NULL);
 
   return &color_primaries[primaries];
 }
@@ -309,8 +318,8 @@ gst_video_color_primaries_get_info (GstVideoColorPrimaries primaries)
 /**
  * gst_video_color_matrix_get_Kr_Kb:
  * @matrix: a #GstVideoColorMatrix
- * @Kr: result red channel coefficient
- * @Kb: result blue channel coefficient
+ * @Kr: (out): result red channel coefficient
+ * @Kb: (out): result blue channel coefficient
  *
  * Get the coefficients used to convert between Y'PbPr and R'G'B' using @matrix.
  *
