@@ -204,7 +204,7 @@ scaler_dump (GstVideoScaler * scale)
  * pixel. If n_taps is 0, this function chooses a good value automatically based
  * on the @method and @in_size/@out_size.
  *
- * Returns: a #GstVideoResample
+ * Returns: a #GstVideoScaler
  */
 GstVideoScaler *
 gst_video_scaler_new (GstVideoResamplerMethod method, GstVideoScalerFlags flags,
@@ -1492,6 +1492,9 @@ gst_video_scaler_2d (GstVideoScaler * hscale, GstVideoScaler * vscale,
       realloc_tmplines (vscale, n_elems, width);
 
     v_taps = vscale->resampler.max_taps;
+    if (vscale->flags & GST_VIDEO_SCALER_FLAG_INTERLACED)
+      v_taps *= 2;
+
     lines = g_alloca (v_taps * sizeof (gpointer));
 
     if (hscale == NULL) {
