@@ -19,6 +19,7 @@
 
 /**
  * SECTION:element-audiorate
+ * @title: audiorate
  * @see_also: #GstVideoRate
  *
  * This element takes an incoming stream of timestamped raw audio frames and
@@ -48,19 +49,20 @@
  * that the incoming data is then simply shifted (by less than the indicated
  * tolerance) to a perfect time.
  *
- * <refsect2>
- * <title>Example pipelines</title>
+ * ## Example pipelines
  * |[
  * gst-launch-1.0 -v autoaudiosrc ! audiorate ! audioconvert ! wavenc ! filesink location=alsa.wav
- * ]| Capture audio from the sound card and turn it into a perfect stream
+ * ]|
+ *  Capture audio from the sound card and turn it into a perfect stream
  * for saving in a raw audio file.
  * |[
  * gst-launch-1.0 -v uridecodebin uri=file:///path/to/audio.file ! audiorate ! audioconvert ! wavenc ! filesink location=alsa.wav
- * ]| Decodes an audio file and transforms it into a perfect stream for saving
+ * ]|
+ *  Decodes an audio file and transforms it into a perfect stream for saving
  * in a raw audio WAV file. Without the audio rate, the timing might not be
  * preserved correctly in the WAV file in case the decoded stream is jittery
  * or there are samples missing.
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -190,10 +192,10 @@ gst_audio_rate_class_init (GstAudioRateClass * klass)
       "Drops/duplicates/adjusts timestamps on audio samples to make a perfect stream",
       "Wim Taymans <wim@fluendo.com>");
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_audio_rate_sink_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_audio_rate_src_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_audio_rate_sink_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_audio_rate_src_template);
 
   element_class->change_state = gst_audio_rate_change_state;
 }
@@ -214,12 +216,24 @@ static gboolean
 gst_audio_rate_setcaps (GstAudioRate * audiorate, GstCaps * caps)
 {
   GstAudioInfo info;
+  gint prev_rate = 0;
 
   if (!gst_audio_info_from_caps (&info, caps))
     goto wrong_caps;
 
+  prev_rate = audiorate->info.rate;
   audiorate->info = info;
 
+  if (audiorate->next_offset >= 0 && prev_rate > 0 && prev_rate != info.rate) {
+    GST_DEBUG_OBJECT (audiorate,
+        "rate changed from %d to %d", prev_rate, info.rate);
+
+    /* calculate next_offset based on new rate value */
+    audiorate->next_offset =
+        gst_util_uint64_scale_int_round (audiorate->next_ts,
+        info.rate, GST_SECOND);
+  }
+
   return TRUE;
 
   /* ERRORS */
@@ -437,6 +451,7 @@ gst_audio_rate_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
   GstFlowReturn ret = GST_FLOW_OK;
   GstClockTimeDiff diff;
   gint rate, bpf;
+  GstAudioMeta *meta;
 
   audiorate = GST_AUDIO_RATE (parent);
 
@@ -490,8 +505,9 @@ gst_audio_rate_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
     in_time = audiorate->next_ts;
   }
 
+  meta = gst_buffer_get_audio_meta (buf);
   in_size = gst_buffer_get_size (buf);
-  in_samples = in_size / bpf;
+  in_samples = meta ? meta->samples : in_size / bpf;
   audiorate->in += in_samples;
 
   /* calculate the buffer offset */
@@ -547,6 +563,10 @@ gst_audio_rate_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
           fillmap.size);
       gst_buffer_unmap (fill, &fillmap);
 
+      if (audiorate->info.layout == GST_AUDIO_LAYOUT_NON_INTERLEAVED) {
+        gst_buffer_add_audio_meta (fill, &audiorate->info, cursamples, NULL);
+      }
+
       GST_DEBUG_OBJECT (audiorate, "inserting %" G_GUINT64_FORMAT " samples",
           cursamples);
 
@@ -589,7 +609,7 @@ gst_audio_rate_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
   } else if (in_offset < audiorate->next_offset) {
     /* need to remove samples */
     if (in_offset_end <= audiorate->next_offset) {
-      guint64 drop = in_size / bpf;
+      guint64 drop = in_samples;
 
       audiorate->drop += drop;
 
@@ -605,24 +625,16 @@ gst_audio_rate_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
 
       goto beach;
     } else {
-      guint64 truncsamples;
-      guint truncsize, leftsize;
-      GstBuffer *trunc;
+      guint64 truncsamples, leftsamples;
 
       /* truncate buffer */
       truncsamples = audiorate->next_offset - in_offset;
-      truncsize = truncsamples * bpf;
-      leftsize = in_size - truncsize;
+      leftsamples = in_samples - truncsamples;
 
-      trunc =
-          gst_buffer_copy_region (buf, GST_BUFFER_COPY_ALL, truncsize,
-          leftsize);
-
-      gst_buffer_unref (buf);
-      buf = trunc;
+      buf = gst_audio_buffer_truncate (buf, bpf, truncsamples, leftsamples);
 
       audiorate->drop += truncsamples;
-      audiorate->out += (leftsize / bpf);
+      audiorate->out += leftsamples;
       GST_DEBUG_OBJECT (audiorate, "truncating %" G_GUINT64_FORMAT " samples",
           truncsamples);
 
