@@ -1078,7 +1078,7 @@ get_compatible_muxer_sink_pad (GstEncodeBin * ebin, GstElement * encoder,
         sinkcaps);
     g_assert (srctempl != NULL);
     sinktempl = gst_element_get_compatible_pad_template (ebin->muxer, srctempl);
-    g_object_unref (srctempl);
+    gst_object_unref (srctempl);
   }
 
   if (G_UNLIKELY (sinktempl == NULL))
@@ -1130,7 +1130,7 @@ _capsfilter_force_format (GstPad * pad,
 
   structure = gst_caps_get_structure (caps, 0);
   gst_structure_remove_field (structure, "streamheader");
-  GST_DEBUG_OBJECT (pad, "Forcing caps to %" GST_PTR_FORMAT, caps);
+  GST_INFO_OBJECT (pad, "Forcing caps to %" GST_PTR_FORMAT, caps);
   g_object_set (GST_OBJECT_PARENT (pad), "caps", caps, NULL);
   g_signal_handler_disconnect (pad, *signal_id);
   *signal_id = 0;
@@ -1168,8 +1168,8 @@ _post_missing_plugin_message (GstEncodeBin * ebin, GstEncodingProfile * prof)
   /* missing plugin support */
   gst_element_post_message (GST_ELEMENT_CAST (ebin),
       gst_missing_encoder_message_new (GST_ELEMENT_CAST (ebin), format));
-  GST_ELEMENT_ERROR (ebin, CORE, MISSING_PLUGIN, (NULL),
-      ("Couldn't create encoder for format %" GST_PTR_FORMAT, format));
+  GST_ELEMENT_ERROR (ebin, CORE, MISSING_PLUGIN,
+      ("Couldn't create encoder for format %" GST_PTR_FORMAT, format), (NULL));
 
   gst_caps_unref (format);
 }
@@ -1383,7 +1383,7 @@ _create_stream_group (GstEncodeBin * ebin, GstEncodingProfile * sprof,
       sinkpad = gst_element_get_static_pad (sgroup->smartencoder, "sink");
     }
     gst_caps_unref (tmpcaps);
-    g_object_unref (srcpad);
+    gst_object_unref (srcpad);
   }
 
   srcpad =
@@ -1395,8 +1395,8 @@ _create_stream_group (GstEncodeBin * ebin, GstEncodingProfile * sprof,
   /* Go straight to splitter */
   if (G_UNLIKELY (fast_pad_link (srcpad, sinkpad) != GST_PAD_LINK_OK))
     goto passthrough_link_failure;
-  g_object_unref (sinkpad);
-  g_object_unref (srcpad);
+  gst_object_unref (sinkpad);
+  gst_object_unref (srcpad);
   srcpad = NULL;
 
   /* Path 2 : Conversion / Encoding */
@@ -1416,8 +1416,8 @@ _create_stream_group (GstEncodeBin * ebin, GstEncodingProfile * sprof,
     srcpad = gst_element_get_static_pad (sgroup->encoder, "src");
     if (G_UNLIKELY (fast_pad_link (srcpad, sinkpad) != GST_PAD_LINK_OK))
       goto encoder_link_failure;
-    g_object_unref (sinkpad);
-    g_object_unref (srcpad);
+    gst_object_unref (sinkpad);
+    gst_object_unref (srcpad);
     srcpad = NULL;
   } else if (gst_encoding_profile_get_preset (sgroup->profile)
       || gst_encoding_profile_get_preset_name (sgroup->profile)) {
@@ -1589,8 +1589,8 @@ _create_stream_group (GstEncodeBin * ebin, GstEncodingProfile * sprof,
     goto no_splitter_srcpad;
   if (G_UNLIKELY (fast_pad_link (srcpad, sinkpad) != GST_PAD_LINK_OK))
     goto splitter_encoding_failure;
-  g_object_unref (sinkpad);
-  g_object_unref (srcpad);
+  gst_object_unref (sinkpad);
+  gst_object_unref (srcpad);
   srcpad = NULL;
 
   /* End of Stream 2 setup */
@@ -1973,8 +1973,8 @@ no_muxer:
     /* missing plugin support */
     gst_element_post_message (GST_ELEMENT_CAST (ebin),
         gst_missing_encoder_message_new (GST_ELEMENT_CAST (ebin), format));
-    GST_ELEMENT_ERROR (ebin, CORE, MISSING_PLUGIN, (NULL),
-        ("No available muxer for format %" GST_PTR_FORMAT, format));
+    GST_ELEMENT_ERROR (ebin, CORE, MISSING_PLUGIN,
+        ("No available muxer for format %" GST_PTR_FORMAT, format), (NULL));
     if (format)
       gst_caps_unref (format);
     return FALSE;
