@@ -22,17 +22,18 @@
 
 /**
  * SECTION:element-videoconvert
+ * @title: videoconvert
  *
  * Convert video frames between a great variety of video formats.
  *
- * <refsect2>
- * <title>Example launch line</title>
+ * ## Example launch line
  * |[
  * gst-launch-1.0 -v videotestsrc ! video/x-raw,format=YUY2 ! videoconvert ! autovideosink
- * ]| This will output a test video (generated in YUY2 format) in a video
+ * ]|
+ *  This will output a test video (generated in YUY2 format) in a video
  * window. If the video sink selected does not support YUY2 videoconvert will
  * automatically convert the video to a format understood by the video sink.
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -67,6 +68,7 @@ G_DEFINE_TYPE (GstVideoConvert, gst_video_convert, GST_TYPE_VIDEO_FILTER);
 #define DEFAULT_PROP_MATRIX_MODE GST_VIDEO_MATRIX_MODE_FULL
 #define DEFAULT_PROP_GAMMA_MODE GST_VIDEO_GAMMA_MODE_NONE
 #define DEFAULT_PROP_PRIMARIES_MODE GST_VIDEO_PRIMARIES_MODE_NONE
+#define DEFAULT_PROP_N_THREADS 1
 
 enum
 {
@@ -79,7 +81,8 @@ enum
   PROP_CHROMA_MODE,
   PROP_MATRIX_MODE,
   PROP_GAMMA_MODE,
-  PROP_PRIMARIES_MODE
+  PROP_PRIMARIES_MODE,
+  PROP_N_THREADS
 };
 
 #define CSP_VIDEO_CAPS GST_VIDEO_CAPS_MAKE (GST_VIDEO_FORMATS_ALL) ";" \
@@ -368,7 +371,13 @@ static gboolean
 gst_video_convert_filter_meta (GstBaseTransform * trans, GstQuery * query,
     GType api, const GstStructure * params)
 {
-  /* propose all metadata upstream */
+  /* This element cannot passthrough the crop meta, because it would convert the
+   * wrong sub-region of the image, and worst, our output image may not be large
+   * enough for the crop to be applied later */
+  if (api == GST_VIDEO_CROP_META_API_TYPE)
+    return FALSE;
+
+  /* propose all other metadata upstream */
   return TRUE;
 }
 
@@ -464,7 +473,9 @@ gst_video_convert_set_info (GstVideoFilter * filter,
           GST_VIDEO_CONVERTER_OPT_GAMMA_MODE,
           GST_TYPE_VIDEO_GAMMA_MODE, space->gamma_mode,
           GST_VIDEO_CONVERTER_OPT_PRIMARIES_MODE,
-          GST_TYPE_VIDEO_PRIMARIES_MODE, space->primaries_mode, NULL));
+          GST_TYPE_VIDEO_PRIMARIES_MODE, space->primaries_mode,
+          GST_VIDEO_CONVERTER_OPT_THREADS, G_TYPE_UINT,
+          space->n_threads, NULL));
   if (space->convert == NULL)
     goto no_convert;
 
@@ -576,6 +587,10 @@ gst_video_convert_class_init (GstVideoConvertClass * klass)
           "Primaries Conversion Mode", gst_video_primaries_mode_get_type (),
           DEFAULT_PROP_PRIMARIES_MODE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (gobject_class, PROP_N_THREADS,
+      g_param_spec_uint ("n-threads", "Threads",
+          "Maximum number of threads to use", 0, G_MAXUINT,
+          DEFAULT_PROP_N_THREADS, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -590,6 +605,7 @@ gst_video_convert_init (GstVideoConvert * space)
   space->matrix_mode = DEFAULT_PROP_MATRIX_MODE;
   space->gamma_mode = DEFAULT_PROP_GAMMA_MODE;
   space->primaries_mode = DEFAULT_PROP_PRIMARIES_MODE;
+  space->n_threads = DEFAULT_PROP_N_THREADS;
 }
 
 void
@@ -628,6 +644,9 @@ gst_video_convert_set_property (GObject * object, guint property_id,
     case PROP_DITHER_QUANTIZATION:
       csp->dither_quantization = g_value_get_uint (value);
       break;
+    case PROP_N_THREADS:
+      csp->n_threads = g_value_get_uint (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -670,6 +689,9 @@ gst_video_convert_get_property (GObject * object, guint property_id,
     case PROP_DITHER_QUANTIZATION:
       g_value_set_uint (value, csp->dither_quantization);
       break;
+    case PROP_N_THREADS:
+      g_value_set_uint (value, csp->n_threads);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
